import json

from os import path, system

from canaryd.packages import click

from canaryd.log import logger
from canaryd.plugin import get_plugin_state, get_plugins, prepare_plugin
from canaryd.remote import CanaryJSONEncoder, register, signup
from canaryd.settings import CanarydSettings, get_config_file, write_settings_to_config

from .install_service import install_service


def init_command(key, auto_start=False):
    config_file = get_config_file()

    # Register if the config file cannot be found
    if not path.exists(config_file):
        did_register = register_command(key)

        if not did_register:
            raise TypeError('Failed to register')

    # Install the service
    click.echo('--> Installing canaryd service')
    start_command = install_service()

    if (
        auto_start or
        click.confirm(
            'Start canaryd service ({0})?'.format(start_command),
            default=True,
        )
    ):
        system(start_command)


def register_command(key):
    config_file = get_config_file()

    if not key:
        click.echo('--> No key provided or set for this instance')
        click.echo('--> To sign up, input your email address, or blank to skip')
        email_or_blank = raw_input('Email or blank: ')

        if '@' not in email_or_blank:
            return False

        # Signup and get the key
        key = signup(email_or_blank)
        click.echo('--> You are now signed up for servicecanary.com')
        click.echo('--> Check your email for a login link to view updates')

    # Register the server
    server_id = register(key=key)

    # Create our settings
    settings = CanarydSettings(api_key=key, server_id=server_id)

    # Write the settings to the config file
    write_settings_to_config(settings)

    click.echo('--> {0} written'.format(config_file))

    return True


def state_command(plugin_name):
    plugins = get_plugins()

    target_plugin = None

    for plugin in plugins:
        if plugin.name == plugin_name:
            target_plugin = plugin
            break

    if not target_plugin:
        raise TypeError('Invalid plugin: {0}'.format(plugin_name))

    prepare_plugin(target_plugin)

    click.echo('State for {0}:'.format(plugin_name))

    status, data = get_plugin_state(target_plugin)

    if status:
        print(json.dumps(
            target_plugin.serialise_state(data),
            cls=CanaryJSONEncoder,
            indent=4,
        ))
    else:
        logger.critical((
            'Unexpected exception while getting {0} state: '
            '{1}({2})'
        ).format(plugin.name, data.__class__.__name__, data))


def plugins_command():
    click.echo('--> Available plugins: {0}'.format(', '.join(
        plugin.name for plugin in get_plugins()
    )))



# canaryd
# File: canaryd/ctl/__main__.py
# Desc: entry point for canaryctl

from __future__ import print_function

from canaryd.packages import click

from canaryd.exceptions import CanarydError
from canaryd.log import logger, setup_logging
from canaryd.remote import ApiError
from canaryd.version import __version__

from . import init_command, register_command, state_command, plugins_command


def _handle_exceptions(func, *args, **kwargs):
    try:
        func(*args, **kwargs)

    except ApiError as e:
        logger.critical('API {0} error: {1}'.format(e.status_code, e.message))

    except CanarydError:
        raise

    except Exception:
        logger.critical('Unexpected exception:')
        raise


# Parse arguments
@click.group()
@click.option('-v', '--verbose', count=True)
def main(verbose=0):
    '''
    canaryd control.
    '''

    # For canaryctl we want warnings to show, so always bump verbosity
    verbose += 1
    setup_logging(verbose)


@main.command()
@click.argument('key', required=False)
def register(key):
    '''
    Register this server on Service Canary.

    If no api key is provided, you can sign up instantly.
    '''

    _handle_exceptions(register_command, key)


@main.command()
@click.option('--start', is_flag=True, default=False)
@click.argument('key', required=False)
def init(start, key):
    '''
    Create the canaryd service and start it.

    This command will attempt to register if the config file is not found.
    '''

    _handle_exceptions(
        init_command,
        key,
        auto_start=start,
    )


@main.command()
@click.argument('plugin')
def state(plugin):
    '''
    Get state for a single plugin.
    '''

    _handle_exceptions(state_command, plugin)


@main.command()
def plugins():
    '''
    List all plugins.
    '''

    _handle_exceptions(plugins_command)


@main.command()
def version():
    '''
    Print the canaryd version.
    '''

    click.echo('canaryd v{0}'.format(__version__))


main()
